/*
 * Galactic Bloodshed, copyright (c) 1989 by Robert P. Chansky, 
 * smq@ucscb.ucsc.edu, mods by people in GB_copyright.h.
 * Restrictions in GB_copyright.h.
 *
 * maxsupport() -- return how many people one sector can support
 * Distmap() -- map-oriented distance that does not look square
 * compatibility() -- return how much race is compatible with planet
 * gravity() -- return gravity for planet
 * prin_ship_orbits() -- prints place ship orbits
 */

#include "GB_copyright.h"
#define EXTERN extern
#include "vars.h"
#include "races.h"
#include "ships.h"
#include <math.h>

/* max this sector can support of specified race.
 */
int maxsupport(r,p,c,toxic)
reg racetype *r;
reg sectortype *p;
float c;	/* (compatibility of race sector) */
int toxic;
{
int val;
float a,b;
a=FACTOR_FERT_SUPPORT * (log1p((double)1.*p->eff)  * log1p((double)p->fert)+1.0);
b=.01 * ((r==NULL) ? 0.0 : r->likes[p->des])
    * (MAX(0.0, MIN(100.0, c+20.0*(r->number_sexes-2)))); 


val = (int)( a * b * .01 * (100.0 - (float)toxic) );

return val;
}



int Distmap(x,y,x2,y2)
reg int x,y,x2,y2;
{
 return( round_rand(fabs((double)x-x2) + fabs( (double)y-y2)*(RATIOXY*0.5) ) );
}


float compatibility(planet, race)
reg planettype *planet;
reg racetype *race;
{
 reg int i,add;
 float sum=1.0;

	/* make an adjustment for planetary temperature */
	add = 0.1*((float)planet->conditions[TEMP]-race->conditions[TEMP]);
	sum *= 1.0 -(abs(add)/100.0);

	/* step through and report compatibility of each planetary gas */
  for (i=TEMP+1; i<=OTHER; i++) {
	add = ((float)planet->conditions[i] - race->conditions[i]);
	sum *= 1.0 - (abs((float)add)/100.0);
  }
  if(sum < 0.0) sum = 0.0;
	sum *= 100.0 - planet->conditions[TOXIC];

  return (sum);

}



float gravity(p)
planettype *p;
{
 return (float)p->Maxx * p->Maxy * GRAV_FACTOR;
}



char Dispshiporbits_buf[PLACENAMESIZE+13];

char *prin_ship_orbits(s)
shiptype *s;
{

  switch (s->whatorbits) {
	case LEVEL_UNIV:
		sprintf(Dispshiporbits_buf,"/(%.0f,%.0f)",s->xpos,s->ypos);
		break;
	case LEVEL_STAR:
		sprintf(Dispshiporbits_buf,"/%s", Stars[s->storbits]->name);
		break;
	case LEVEL_PLAN:
		sprintf(Dispshiporbits_buf,"/%s/%s", 
			Stars[s->storbits]->name,
			Stars[s->storbits]->pnames[s->pnumorbits]);
		break;
	default:
		break;
  }
  return Dispshiporbits_buf;
}
