/*
 * Galactic Bloodshed, copyright (c) 1989 by Robert P. Chansky, 
 * smq@ucscb.ucsc.edu, mods by people in GB.c, enroll.dat.
 * Restrictions in GB.c.
 */


#define STYPE_POD	0
#define STYPE_SHUTTLE 	1
#define STYPE_CARRIER   2
#define STYPE_DREADNT	3
#define STYPE_BATTLE 	4
#define STYPE_INTCPT	5
#define STYPE_CRUISER	6
#define STYPE_DESTROYER 7
#define STYPE_FIGHTER 	8
#define STYPE_EXPLORER 	9
#define STYPE_HABITAT 	10
#define STYPE_STATION	11
#define STYPE_ASS	12
#define STYPE_CARGO 	13
#define STYPE_TANKER 	14
#define STYPE_GOD	15
#define STYPE_MINE	16
#define STYPE_MIRROR	17

#define OTYPE_STELE	18
#define OTYPE_GTELE	19
#define OTYPE_TRACT	20
#define OTYPE_AP	21
#define OTYPE_CANIST	22
#define OTYPE_VN	23
#define OTYPE_BERS	24
#define OTYPE_GOV	25
#define OTYPE_OMCL	26
#define OTYPE_TOXWC	27
#define OTYPE_PROBE	28
#define OTYPE_GR	29
#define OTYPE_FACTORY	30
#define OTYPE_TERRA	31
#define OTYPE_BERSCTLC	32
#define OTYPE_AUTOFAC	33
#define OTYPE_TRANSDEV	34
#define OTYPE_REPAIR	35
#define OTYPE_PLANDEF	36

#define ABIL_CARGO 	0
#define ABIL_DESTCAP 	1
#define ABIL_GUNS 	2
#define ABIL_FUELCAP 	3
#define ABIL_TECH 	4
#define ABIL_MAXCREW 	5
#define ABIL_ARMOR 	6
#define ABIL_TARGET	7
#define ABIL_COST 	8
#define ABIL_JUMP 	9
#define ABIL_CANLAND	10
#define ABIL_HASSWITCH	11
#define ABIL_SPEED	12
#define ABIL_DAMAGE	13
#define ABIL_BUILD	14
#define ABIL_CONSTRUCT	15
#define ABIL_SIGHT	16
#define ABIL_PROGRAMMED 17

#define NUMSTYPES 	(OTYPE_PLANDEF+1)
#define NUMABILS 	(ABIL_PROGRAMMED+1)

#define SHIP_NAMESIZE	18



typedef struct ship shiptype;
typedef struct ship_new shiptype_new;
typedef struct place placetype;


struct ship {
    float xpos,ypos;
    float fuel,mass;
    char name[SHIP_NAMESIZE];		/* name of ship (optional) */
    char class[SHIP_NAMESIZE];		/* class of ship - designated by players */
    percent morale;			/* morale of the crew */
    unsigned short command;		/* captain+staff leadership value */

    unsigned short destshipno;		/* destination ship # */
    unsigned short nextship;		/* next ship in linked list */

    unsigned bombard : 1;	/* bombard planet we're orbiting */

    percent armor;
    percent guns;
    percent size;

    unsigned short max_crew;
    unsigned short max_resource;
    unsigned short max_destruct;
    unsigned short max_fuel;
    unsigned short max_speed;
    unsigned short build_type;	/* for factories - type of ship it makes */
    unsigned short build_cost; /* how much it cost to build the ship - into VPs too? */

    float base_mass;
    float tech;		/* technology rating */

	struct a {		/* if the ship is a Space Mirror */
		unsigned short shipno;	/* aimed at what ship */
		char snum;		/* aimed at what star */
		percent intensity;		/* intensity of aiming */
		unsigned pnum : BITS_MAXPLANETS;  /* aimed at what planet */
		unsigned level : 2;	/* aimed at what level */
	} aimed_at;
	
	struct o2 {
		short number;	/* for misc functions */
		short number2;	/* again */
		short number3;
		short number4;  /* again */
	} object;
	
	struct m {
		unsigned mode : 1; /* 0 for radiation, 1 for damage */
	} mine;
	
	struct nv {
		unsigned on : 1;    /* toggles navigate mode */
		unsigned speed : 3; /* speed for navigate command */
		unsigned turns : 16;/* number turns left in maneuver */
		unsigned bearing : 9; /* course */
	} navigate;
	
	struct prot {
		float maxrng;	       /* maximum range for autoshoot */
		unsigned on	 :  1; /* toggle on/off */
		unsigned planet  :  1; /* planet defender */
 		unsigned self	 :  1; /* retaliate if attacked */
 		unsigned evade	 :  1; /* evasive action */
 		unsigned ship	 : 14; /* ship it is protecting */
	} protect;

    struct hd {
    unsigned char charge;
    unsigned ready : 1;
    unsigned on : 1;
    unsigned dummy : 6;  /* dummy bits for development */
	} hyper_drive;

    char storbits;		/* what star # orbits */
    char deststar;		/* destination star */
    unsigned destpnum : BITS_MAXPLANETS;	/* destination planet */
    unsigned pnumorbits : BITS_MAXPLANETS;	/* # of planet if orbiting */
    unsigned damage : 7;		/* amt of damage */
    unsigned rad : 7;			/* radiation level */
    unsigned active: 1;			/* tells whether the ship is active */
    unsigned short destruct;	 	/* stuff it's carrying */
    unsigned short resource;
    unsigned short popn;			/* crew */
    unsigned owner : BITS_MAXPLAYERS;	/* owner of ship */
    unsigned speed : 4;		/* what speed to travel at 0-9 */
    unsigned whatdest : 2;	/* where going (same as Dir) */
    unsigned whatorbits : 2;	/* where orbited (same as Dir) */
    unsigned type : 6;		/* what type ship is */
    unsigned is_cloaked : 1;	/* 1 bit: is cloaked ship */
    unsigned is_sheep : 1;	/* 1 bit: is under influence of mind control */
    unsigned is_docked : 1;	/* 1 bit: is landed on a planet or docked */
    unsigned notified : 1; 	/* 1bit: has been notified of something */
    unsigned is_alive : 1;	/* 1 bit: ship is alive */
    unsigned is_examined : 1;	/* 1 bit: has been examined */
    unsigned on : 1;		/* on or off */
};


struct place {		/* used in function return for finding place */
    char snum;
    char pnum;
    short shipno;
    shiptype *shipptr;
    unsigned level : 2;	/* .level: same as Dir */
    unsigned err : 1;		/* if error */
};

 /* whether ship is an object or not */
#define is_object(s) ((s)->type > STYPE_MIRROR)

 /* can takeoff & land, is mobile, etc. */
#define speed_rating(s) (Shipdata[(s)->type][ABIL_SPEED])

 /* has an on/off switch */
#define has_switch(s) (Shipdata[(s)->type][ABIL_HASSWITCH])

 /* can bombard planets */
#define can_bombard(s) \
	(Shipdata[(s)->type][ABIL_GUNS] && ((s)->type != STYPE_MINE))

/* can navigate */
#define can_navigate(s) (Shipdata[(s)->type][ABIL_SPEED] > 0 && (s)->type != OTYPE_TERRA && (s)->type != OTYPE_VN)

 /* can aim at things. */
#define can_aim(s) ((s)->type>=STYPE_MIRROR && (s)->type<=OTYPE_TRACT)

 /* macro for killing a ship.  !notified means we have not yet written it
    to the dead_ship file.  */
#define kill_ship(s) ((s)->is_alive = 0, (s)->damage = 100, (s)->notified = 0)

/* macros to get ship stats */
#define Armor(s) ( ((s)->type==OTYPE_FACTORY) ? Shipdata[(s)->type][ABIL_ARMOR] : (s)->armor)
#define Guns(s) ( ((s)->type==OTYPE_FACTORY) ? Shipdata[(s)->type][ABIL_GUNS] : (s)->guns)
#define Size(s) ( ((s)->type==OTYPE_FACTORY) ? Shipdata[(s)->type][ABIL_TARGET] : (s)->size)
#define Max_crew(s) ( ((s)->type==OTYPE_FACTORY) ? Shipdata[(s)->type][ABIL_MAXCREW] : (s)->max_crew)
#define Max_resource(s) ( ((s)->type==OTYPE_FACTORY) ? Shipdata[(s)->type][ABIL_CARGO] : (s)->max_resource)
#define Max_fuel(s) ( ((s)->type==OTYPE_FACTORY) ? Shipdata[(s)->type][ABIL_FUELCAP] : (s)->max_fuel)
#define Max_destruct(s) ( ((s)->type==OTYPE_FACTORY) ? Shipdata[(s)->type][ABIL_DESTCAP] : (s)->max_destruct)
#define Max_speed(s) ( ((s)->type==OTYPE_FACTORY) ? Shipdata[(s)->type][ABIL_SPEED] : (s)->max_speed)
#define Cost(s) ( ((s)->type==OTYPE_FACTORY) ? Shipdata[(s)->type][ABIL_COST ] : (s)->build_cost)
#define Mass(s) ( ((s)->type==OTYPE_FACTORY) ? (s)->mass : (s)->base_mass)

extern placetype Getplace();
extern char *Dispplace(),*Dispshiporbits(),*prin_ship_orbits(),
	*prin_aimed_at(), *prin_ship_dest();
extern int Shipdata[NUMSTYPES][NUMABILS];
extern char Shipltrs[];
extern char *Shipnames[];
extern float tele_range(), gun_range();

EXTERN shiptype **ships;
EXTERN shiptype *shipptr;
